import { OUTPUT_DATETIME_TRANSFORMER } from '@modules/ad/ad.di-tokens';
import { AdMapper } from '@modules/ad/ad.mapper';
import { DateTimeTransformerPort } from '@modules/ad/core/application/ports/datetime-transformer.port';
import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import { Frequency, Status } from '@modules/ad/core/domain/ad.types';
import {
  AdReadModel,
  AdWriteModel,
} from '@modules/ad/infrastructure/ad.repository';
import { AdResponseDto } from '@modules/ad/interface/dtos/ad.response.dto';
import { Test } from '@nestjs/testing';

const now = new Date('2023-06-21 06:00:00');
const adEntity: AdEntity = new AdEntity({
  id: 'c160cf8c-f057-4962-841f-3ad68346df44',
  props: {
    userId: '7ca2490b-d04d-4ac5-8d6c-5c416fab922e',
    driver: false,
    passenger: true,
    status: Status.PENDING,
    frequency: Frequency.PUNCTUAL,
    fromDate: '2023-06-21',
    toDate: '2023-06-21',
    schedule: [
      {
        day: 3,
        time: '07:15',
        margin: 900,
      },
    ],
    waypoints: [
      {
        position: 0,
        address: {
          houseNumber: '5',
          street: 'Avenue Foch',
          locality: 'Nancy',
          postalCode: '54000',
          country: 'France',
          coordinates: {
            lat: 48.689445,
            lon: 6.1765102,
          },
        },
      },
      {
        position: 1,
        address: {
          locality: 'Paris',
          postalCode: '75000',
          country: 'France',
          coordinates: {
            lat: 48.8566,
            lon: 2.3522,
          },
        },
      },
    ],
    strict: false,
    seatsProposed: 3,
    seatsRequested: 1,
  },
  createdAt: now,
  updatedAt: now,
});
const adReadModel: AdReadModel = {
  uuid: 'c160cf8c-f057-4962-841f-3ad68346df44',
  userUuid: '7ca2490b-d04d-4ac5-8d6c-5c416fab922e',
  driver: false,
  passenger: true,
  status: Status.PENDING,
  frequency: Frequency.PUNCTUAL,
  fromDate: new Date('2023-06-21'),
  toDate: new Date('2023-06-21'),
  schedule: [
    {
      uuid: '3978f3d6-560f-4a8f-83ba-9bf5aa9a2d27',
      day: 3,
      time: new Date('2023-06-21T07:05:00Z'),
      margin: 900,
      createdAt: now,
      updatedAt: now,
    },
  ],
  waypoints: [
    {
      uuid: '6f53f55e-2bdb-4c23-b6a9-6d7b498e47b9',
      position: 0,
      houseNumber: '5',
      street: 'Avenue Foch',
      locality: 'Nancy',
      postalCode: '54000',
      country: 'France',
      lat: 48.689445,
      lon: 6.1765102,
      createdAt: now,
      updatedAt: now,
    },
    {
      uuid: 'e18c6a84-0ab7-4e44-af1d-829d0b0d0573',
      position: 1,
      locality: 'Paris',
      postalCode: '75000',
      country: 'France',
      lat: 48.8566,
      lon: 2.3522,
      createdAt: now,
      updatedAt: now,
    },
  ],
  strict: false,
  seatsProposed: 3,
  seatsRequested: 1,
  createdAt: now,
  updatedAt: now,
};

const mockOutputDatetimeTransformer: DateTimeTransformerPort = {
  fromDate: jest.fn(),
  toDate: jest.fn(),
  day: jest.fn(),
  time: jest.fn(),
};

describe('Ad Mapper', () => {
  let adMapper: AdMapper;

  beforeAll(async () => {
    const module = await Test.createTestingModule({
      providers: [
        AdMapper,
        {
          provide: OUTPUT_DATETIME_TRANSFORMER,
          useValue: mockOutputDatetimeTransformer,
        },
      ],
    }).compile();
    adMapper = module.get<AdMapper>(AdMapper);
  });

  it('should be defined', () => {
    expect(adMapper).toBeDefined();
  });

  it('should map domain entity to persistence data', async () => {
    const mapped: AdWriteModel = adMapper.toPersistence(adEntity);
    expect(mapped.waypoints.create[0].uuid.length).toBe(36);
    expect(mapped.waypoints.create[1].uuid.length).toBe(36);
    expect(mapped.schedule.create.length).toBe(1);
  });

  it('should map persisted data to domain entity', async () => {
    const mapped: AdEntity = adMapper.toDomain(adReadModel);
    expect(mapped.getProps().waypoints[0].address.coordinates.lat).toBe(
      48.689445,
    );
    expect(mapped.getProps().waypoints[1].address.coordinates.lon).toBe(2.3522);
    expect(mapped.getProps().schedule.length).toBe(1);
    expect(mapped.getProps().schedule[0].time).toBe('07:05');
  });

  it('should map domain entity to response', async () => {
    const mapped: AdResponseDto = adMapper.toResponse(adEntity);
    expect(mapped.id).toBe('c160cf8c-f057-4962-841f-3ad68346df44');
  });
});
